'+-----------------------------------------------------------------------------+
'|                                                                             |
'| Autor: Jonas Eberhard   www.joseng.de                                       |
'| Datei: DMX LED Dimmer 1Kanal strobe.bas                                     |
'| Beschreibung: LED Dimmer 1 Kanal per DMX mit einer                          |
'|               PWM Frequenz von ca. 30kHz                                    |
'|                                                                             |
'| Prozessor: ATtiny2313                                                       |
'| Fuse Bits:  LowByte: 0xFF    HighByte: 0xDF    EByte: 0xFF                  |
'|                                                                             |
'+-----------------------------------------------------------------------------+

'+-----------------------------------------------------------------------------+
'|                                                                             |
'| Strobe Kanal:                                                               |
'|                                                                             |
'| |---|--------------------------|----|                                       |
'| 0   15                        240  255                                      |
'|                                                                             |
'|     |--------------------------|                                            |
'|               225                                                           |
'|                                                                             |
'|    225/15 = 15  -> 1-15Hz                                                   |
'|                                                                             |
'|    Frequenz = (dmx-15) / 15                                                 |
'|                                                                             |
'|                                                                             |
'+-----------------------------------------------------------------------------+

'+-----------------------------------------------------------------------------+
'|                                                                             |
'| Timercounterwert:                                                           |
'|                                                                             |
'| Timerfrequenz 1kH = 1000Hz                                                  |
'|                                                                             |
'| Frequenz = (dmx-15) / 15 (siehe oberen Kasten)                              |
'|                                                                             |
'| => Timercounterwert = 1000/((dmx-15)/15)                                    |
'|                                                                             |
'|  Umgestellt da Bruch unter Bruchstrich (man teilt durch einen Bruch in dem  |
'|  man mit dem Kehrbruch multipliziert. Die 15 unter dem unteren Bruch        |
'|  kommen hoch zu den 1000, also steht im Nenner 1000 * 15 = 15000)           |
'|                                                                             |
'|  --> Timercounterwert = 15000/(dmx-15)                                      |
'|                                                                             |
'+-----------------------------------------------------------------------------+


$regfile = "attiny2313.dat"                                 'ATtiny2313
$crystal = 16000000                                         '16MHz Quarz
$baud = 250000                                              'Baudrate 250Kbaud


Config Timer0 = Timer , Prescale = 64
On Timer0 Strobe

Config Timer1 = Pwm , Pwm = 8 , Prescale = 1 , Compare A Pwm = Clear Down , Compare B Pwm = Clear Down


'Die Timer starten
Enable Timer0
Enable Timer1


'Subs
Declare Sub Adresse_basteln()

'IOs
Config Portd.4 = Input                                      'Adresse Bit 0
Config Portd.3 = Input                                      'Adresse Bit 1
Config Portd.6 = Input                                      'Adresse Bit 2
Config Portb.0 = Input                                      'Adresse Bit 3
Config Portb.1 = Input                                      'Adresse Bit 4
Config Portb.4 = Input                                      'Adresse Bit 5
Config Portb.5 = Input                                      'Adresse Bit 6
Config Portb.6 = Input                                      'Adresse Bit 7
Config Portb.7 = Input                                      'Adresse Bit 8

Config Portd.1 = Input                                      'DIP-Switch 10 -> Stand Alone Mode

Config Portb.3 = Output                                     'PWM Out

Config Portd.2 = Output                                     'Statusled


'PullUps setzten
Set Portd.4
Set Portd.3
Set Portd.6
Set Portb.0
Set Portb.1
Set Portb.4
Set Portb.5
Set Portb.6
Set Portb.7
Set Portd.1


'Aliases
Statusled Alias Portd.2

Adressbit0 Alias Pind.4
Adressbit1 Alias Pind.3
Adressbit2 Alias Pind.6
Adressbit3 Alias Pinb.0
Adressbit4 Alias Pinb.1
Adressbit5 Alias Pinb.4
Adressbit6 Alias Pinb.5
Adressbit7 Alias Pinb.6
Adressbit8 Alias Pinb.7

Alonedip Alias Pind.1

Pwm_out Alias Pwm1a


'Funktionsvariablen
Const Timervorgabe0 = 6
Dim Adresse As Integer                                      'Variable zum zusammenbasteln der DMX-Adresse
Dim Pwmwert As Byte
Dim Strobewert As Byte
Dim Timercounter As Integer
Dim Timercounterwert As Integer
Dim Dummy As Integer
Dim Standalone As Byte                                      'Zeigt an, dass sich der DMX-LED-Dimmer im Stand Alone Mode befindet
Dim Dummybyte As Byte

'DMX Variablen
Dim Dmx_count As Word                                       'Pakete die auf Ausgnge zielen
Dim Dmx_count_temp As Byte                                  'Gesendete Pakete
Dim Dmx_byte As Byte                                        'DMX Wert
Dim Dmx_clock As Byte                                       'Spielerei
Dim Dmx_adr As Word                                         'DMX-Adresse
Dim Dmx_adr_temp As Byte                                    'DMX-Adresse (Temp)
Dim Dmx_adr_c As Word                                       'DMX-Adresse (Counter)
Dim Dmx_off As Byte                                         'DMX-Signal noch da?


'Startwerte setzen
Set Statusled
Pwm_out = 0
Timer0 = Timervorgabe0


' Uart initialiesieren
On Urxc Rec_isr
Enable Urxc

'UART TX Pin ausschalten
Reset Ucsrb.txen

' Anfangsvariablen Festlegen
Dmx_count = 0                                               'kein Paket empfangen
Dmx_clock = 16                                              '

Enable Interrupts

' ******************************************************************************
' Hauptschleife
' ******************************************************************************
Do
  Waitms 1

  Decr Dmx_off

  If Dmx_off = 0 Then
    Reset Statusled
  End If

      'Wenn DIP-Schalter 10 gesetzt dann Stand Alone Modus
  If Alonedip = 0 Then
    Standalone = 1
    If Adresse > 256 Then
      'Noch kein extra Mode wenn DIP 9 ein
    Else
      'Wenn DIP-Schalter 9 in Stand Alone Modus nicht gesetzt, dann kann mit Schaltern 1-4 die Helligkeit eingestellt werden
      Dummybyte = Adresse And &H001F
      Pwmwert = Dummybyte * 8

      Dummybyte = Adresse And &H01E0
      If Dummybyte > 0 Then
        Dummybyte = Dummybyte / 32
        Dummybyte = Dummybyte * 15
        Strobewert = Dummybyte + 15
      Else
        Strobewert = 255
      End If
    End If
  Else
    Standalone = 0
  End If

  Call Adresse_basteln
Loop

' ******************************************************************************
' Funktion: DMX-Signal emfangen und auswerten
'           Dieser Interupt wird beim eintreffen von seriellen Daten ausgefhrt
Rec_isr:
   $asm                                                     'Programmteil fr das Auswerten der
      .def Uart_status = R16                                'emfangenen Daten
      .def Uart_byte = R17
      .def Uart_temp1 = R18

            IN Uart_status,USR
            IN Uart_byte,UDR

            SBRC Uart_status,FE                             'Auswertung ob Datenbyte OK
            RJMP  Asm_reset                                 'bei berlnge des Datenbyte = DMX Reset
            RJMP  Asm_Loop1                                 'Daten OK--->

      Asm_reset:
            LDI Uart_temp1,$00                              'Adesszhler zurcksetzen
            STS {Dmx_count_temp} , Uart_temp1               'Daten dem Basicvariablen bergeben
            STS {Dmx_byte} , Uart_byte
            RJMP Asm_exit
      Asm_loop1:
            STS {Dmx_byte} , Uart_byte
      Asm_exit:
   $end Asm

   If Dmx_count_temp <> 0 Then                              'emfangene Daten im "Dmx_buf" ablegen
      Incr Dmx_count                                        'und den jeweiligen Kanal zuordnen
      If Dmx_adr_c = Dmx_count Then
         If Dmx_adr_temp < 3 Then
            Select Case Dmx_adr_temp
               Case 1:
                 If Standalone = 0 Then
                   Pwmwert = Dmx_byte
                 End If
               Case 2:
                 If Standalone = 0 Then
                   Strobewert = Dmx_byte
                 End If
            End Select
            Incr Dmx_adr_temp
            Incr Dmx_adr_c
         End If
      End If
   Else                                                     'bei DMX Reset Variablen setzen
      Dmx_count = 0 : Dmx_count_temp = 1
      Dmx_adr_c = Dmx_adr + 1 : Dmx_adr_temp = 1
      Decr Dmx_clock
      'nur eine Spielerei Toggelt eine LED am PortD.6 beim eintreffen von korrekten DMX Daten
      If Dmx_clock = 0 Then : Toggle Statusled : Dmx_clock = 16 : End If
   End If
   Dmx_off = 20
Return

Sub Adresse_basteln
  'Adresse zusammenbasteln
  Adresse = 0
  If Adressbit0 = 0 Then Adresse = Adresse + 1
  If Adressbit1 = 0 Then Adresse = Adresse + 2
  If Adressbit2 = 0 Then Adresse = Adresse + 4
  If Adressbit3 = 0 Then Adresse = Adresse + 8
  If Adressbit4 = 0 Then Adresse = Adresse + 16
  If Adressbit5 = 0 Then Adresse = Adresse + 32
  If Adressbit6 = 0 Then Adresse = Adresse + 64
  If Adressbit7 = 0 Then Adresse = Adresse + 128
  If Adressbit8 = 0 Then Adresse = Adresse + 256

  'Adresse speichern
  Dmx_adr = Adresse
End Sub

'Strobe Timer IRQ
Strobe:
  Timer0 = Timervorgabe0

  If Strobewert < 15 Then
    Pwm_out = 0
  Elseif Strobewert > 14 And Strobewert < 241 Then
    Timercounterwert = Strobewert - 15                      'Timercounter berechnung, herleitung Formel siehe Header
    Timercounterwert = 15000 / Timercounterwert
    Dummy = Timercounterwert / 2

    Incr Timercounter

    If Timercounter > Timercounterwert Then                 'Wenn counter ganz oben, Licht wieder aus
        Timercounter = 0
        Pwm_out = 0
    Elseif Timercounter > Dummy Then                        'Wenn die Hlfte erreicht, dann Licht an
      Pwm_out = Pwmwert
    End If
  Elseif Strobewert > 241 Then
    Pwm_out = Pwmwert
  End If

Return